"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuHighlight = void 0;
const Component_1 = require("../Component");
const Viewport_1 = require("../services/Viewport");
/**
 * Manages the sticky state of the navigation and moves the highlight
 * to the current navigation item.
 */
class MenuHighlight extends Component_1.Component {
    /**
     * Create a new MenuHighlight instance.
     *
     * @param options  Backbone view constructor options.
     */
    constructor(options) {
        super(options);
        /**
         * List of all discovered anchors.
         */
        this.anchors = [];
        /**
         * Index of the currently highlighted anchor.
         */
        this.index = -1;
        Viewport_1.Viewport.instance.addEventListener("resize", () => this.onResize());
        Viewport_1.Viewport.instance.addEventListener("scroll", (e) => this.onScroll(e));
        this.createAnchors();
    }
    /**
     * Find all anchors on the current page.
     */
    createAnchors() {
        let base = window.location.href;
        if (base.indexOf("#") != -1) {
            base = base.substr(0, base.indexOf("#"));
        }
        this.el.querySelectorAll("a").forEach((el) => {
            const href = el.href;
            if (href.indexOf("#") == -1)
                return;
            if (href.substr(0, base.length) != base)
                return;
            const hash = href.substr(href.indexOf("#") + 1);
            const anchor = document.querySelector("a.tsd-anchor[name=" + hash + "]");
            const link = el.parentNode;
            if (!anchor || !link)
                return;
            this.anchors.push({
                link: link,
                anchor: anchor,
                position: 0,
            });
        });
        this.onResize();
    }
    /**
     * Triggered after the viewport was resized.
     */
    onResize() {
        let anchor;
        for (let index = 0, count = this.anchors.length; index < count; index++) {
            anchor = this.anchors[index];
            const rect = anchor.anchor.getBoundingClientRect();
            anchor.position = rect.top + document.body.scrollTop;
        }
        this.anchors.sort((a, b) => {
            return a.position - b.position;
        });
        const event = new CustomEvent("scroll", {
            detail: {
                scrollTop: Viewport_1.Viewport.instance.scrollTop,
            },
        });
        this.onScroll(event);
    }
    /**
     * Triggered after the viewport was scrolled.
     *
     * @param event  The custom event with the current vertical scroll position.
     */
    onScroll(event) {
        const scrollTop = event.detail.scrollTop + 5;
        const anchors = this.anchors;
        const count = anchors.length - 1;
        let index = this.index;
        while (index > -1 && anchors[index].position > scrollTop) {
            index -= 1;
        }
        while (index < count && anchors[index + 1].position < scrollTop) {
            index += 1;
        }
        if (this.index != index) {
            if (this.index > -1)
                this.anchors[this.index].link.classList.remove("focus");
            this.index = index;
            if (this.index > -1)
                this.anchors[this.index].link.classList.add("focus");
        }
    }
}
exports.MenuHighlight = MenuHighlight;
