"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Signature = void 0;
const Component_1 = require("../Component");
const Viewport_1 = require("../services/Viewport");
/**
 * Holds a signature and its description.
 */
class SignatureGroup {
    /**
     * Create a new SignatureGroup instance.
     *
     * @param signature    The target signature.
     * @param description  The description for the signature.
     */
    constructor(signature, description) {
        this.signature = signature;
        this.description = description;
    }
    /**
     * Add the given class to all elements of the group.
     *
     * @param className  The class name to add.
     */
    addClass(className) {
        this.signature.classList.add(className);
        this.description.classList.add(className);
        return this;
    }
    /**
     * Remove the given class from all elements of the group.
     *
     * @param className  The class name to remove.
     */
    removeClass(className) {
        this.signature.classList.remove(className);
        this.description.classList.remove(className);
        return this;
    }
}
/**
 * Controls the tab like behaviour of methods and functions with multiple signatures.
 */
class Signature extends Component_1.Component {
    /**
     * Create a new Signature instance.
     *
     * @param options  Backbone view constructor options.
     */
    constructor(options) {
        super(options);
        /**
         * List of found signature groups.
         */
        this.groups = [];
        /**
         * The index of the currently displayed signature.
         */
        this.index = -1;
        this.createGroups();
        if (this.container) {
            this.el.classList.add("active");
            Array.from(this.el.children).forEach((signature) => {
                signature.addEventListener("touchstart", (event) => this.onClick(event));
                signature.addEventListener("click", (event) => this.onClick(event));
            });
            this.container.classList.add("active");
            this.setIndex(0);
        }
    }
    /**
     * Set the index of the active signature.
     *
     * @param index  The index of the signature to activate.
     */
    setIndex(index) {
        if (index < 0)
            index = 0;
        if (index > this.groups.length - 1)
            index = this.groups.length - 1;
        if (this.index == index)
            return;
        const to = this.groups[index];
        if (this.index > -1) {
            const from = this.groups[this.index];
            from.removeClass("current").addClass("fade-out");
            to.addClass("current");
            to.addClass("fade-in");
            Viewport_1.Viewport.instance.triggerResize();
            setTimeout(() => {
                from.removeClass("fade-out");
                to.removeClass("fade-in");
            }, 300);
        }
        else {
            to.addClass("current");
            Viewport_1.Viewport.instance.triggerResize();
        }
        this.index = index;
    }
    /**
     * Find all signature/description groups.
     */
    createGroups() {
        const signatures = this.el.children;
        if (signatures.length < 2)
            return;
        this.container = this.el.nextElementSibling;
        const descriptions = this.container.children;
        this.groups = [];
        for (let index = 0; index < signatures.length; index++) {
            this.groups.push(new SignatureGroup(signatures[index], descriptions[index]));
        }
    }
    /**
     * Triggered when the user clicks onto a signature header.
     *
     * @param e  The related event object.
     */
    onClick(e) {
        this.groups.forEach((group, index) => {
            if (group.signature === e.currentTarget) {
                this.setIndex(index);
            }
        });
    }
}
exports.Signature = Signature;
